import ansible.constants as C
from ansible.executor.task_queue_manager import TaskQueueManager
from ansible.plugins.callback import CallbackBase
import shutil
from takeltest.exceptions import AnsibleRunError


class AnsibleRun(object):

    def __init__(
            self,
            inventory,
            variable_manager,
            loader):
        self._inventory = inventory
        self._variable_manager = variable_manager
        self._loader = loader

    def run(self, play):
        tqm = None
        rc = ResultCallback()
        try:
            tqm = TaskQueueManager(
                inventory=self._inventory,
                variable_manager=self._variable_manager,
                loader=self._loader,
                passwords=dict(),
                stdout_callback=rc)
            tqm.run(play)
        finally:
            if tqm is not None:
                tqm.cleanup()
            shutil.rmtree(C.DEFAULT_LOCAL_TMP, True)

        if rc.failed_playbook_run:
            raise AnsibleRunError(
                'Unable to run playbook.',
                rc.result_playbook_run[-1])
        return rc.result_playbook_run


class ResultCallback(CallbackBase):

    def __init__(self):
        super(ResultCallback, self).__init__()
        self.result_playbook_run = list()
        self.failed_playbook_run = False

    def v2_runner_on_ok(self, result, **kwargs):
        self._clean_results(result._result, result._task.action)
        self.result_playbook_run.append(result._result)

    def v2_runner_on_failed(self, result, *args, **kwargs):
        self._clean_results(result._result, result._task.action)
        error = 'Error message: ' + result._result['msg']
        self.result_playbook_run.append(error)
        self.failed_playbook_run = True



from math import floor
from takeltest.exceptions import TemplatesResolveError


class Templates(object):

    def __init__(self, moleculebook):
        self._moleculebook = moleculebook
        self._templates = list()

    def add(self, jinja2):
        template = dict()
        template['unresolved'] = jinja2
        self._templates.append(template)

    def get_template(self, index):
        return self._templates[index]

    def get_templates(self):
        return self._templates

    def reset(self):
        self._templates = list()

    def resolve(self):
        '''Resolve all variables of a play managed by molecule.'''

        # reset playbook to default
        self._moleculebook.create()

        # populate playbook with task
        self._add_debug_tasks()

        # run playbook to resolve vars
        playbook_results = self._moleculebook.run()

        # scrutinize playbook results
        self._gather_results(playbook_results)

    def _gather_results(self, playbook_results):
        try:

            # delete ansible_facts task result
            del playbook_results[0]

            for index, playbook_result in enumerate(playbook_results):
                template_index = floor(index/2)
                if (index % 2) == 0:
                    if playbook_result['msg'].strip('"') == 'True':
                        self._templates[template_index]['string'] = True
                    else:
                        self._templates[template_index]['string'] = False
                else:
                    template_resolved = playbook_result['msg']
                    self._templates[template_index]['resolved'] = \
                        template_resolved
        except (IndexError, KeyError):
            raise TemplatesResolveError('Unable to resolve jinja2 templates.')

    def _add_debug_tasks(self):
        for template in self._templates:
            unres = template['unresolved']

            # add debug task to test if the template is a string
            self._moleculebook.add_task_debug(
                '"{% if ' + unres + ' | string == ' + unres + ' %}' +
                'True{% else %}False{% endif %}"')

            # add a debug task to resolve jinja2 template
            self._moleculebook.add_task_debug('"{{' + unres + ' | to_json }}"')



import setuptools

if __name__ == '__main__':
    setuptools.setup(
        name='takeltest',
        version='0.7.18',
        author='Geospin',
        author_email='takelage@geospin.de',
        description='Fixtures for ansible, testinfra and molecule',
        long_description='See takelage-var at github: '
                         'https://github.com/geospin-takelage/takelage-var',
        url='https://github.com/geospin-takelage/takelage-var',
        license='GPLv3',
        packages=setuptools.find_packages(),
        package_data={
            '': ['*.md'],
        },
        entry_points={
            'pytest11': ['takeltest=takeltest.plugin']
        },
        install_requires=[
            'ansible>=2.8',
        ],
        platforms='any',
        zip_safe=False,
        classifiers=[
            'Programming Language :: Python',
            'Programming Language :: Python :: 3',
            'Intended Audience :: Information Technology',
            'Intended Audience :: Developers',
            'Intended Audience :: System Administrators',
            'License :: OSI Approved :: MIT License',
            'Operating System :: POSIX',
            'Topic :: Software Development :: Libraries :: Python Modules',
            'Topic :: Software Development :: Testing',
            'Topic :: System :: Systems Administration',
            'Framework :: Pytest',
        ],
    )



from ansible.inventory.manager import InventoryManager
from ansible.parsing.dataloader import DataLoader


class AnsibleInventory(object):

    def __init__(
            self,
            inventoryfile):
        self._inventory = InventoryManager(
            loader=DataLoader(),
            sources=str(inventoryfile))

    def get(self):
        return self._inventory
